------------------------------------------------------------------------------------------
-- Whammy
-- Created By:  Jaycyn of Hyjal
--
-- What is it: Heal, Decurse and more with a point and click
-- License   : Distribute as you wish, please give credit to the author, please send
--           :   bugs and suggestions instead of modifying code yourself.
------------------------------------------------------------------------------------------

W = {}; --this is the namespace for the methods in the Whammy sections of the addon
WC = {}; --this is the namespace for the methods in the WhammyConfig sections of the addon
WP = {}; -- this is the namespace for the methods in the WhammyPet sections of the addon
WS = {}; -- this is the namespace for the methods in the WhammySummary sections of the addon
WB = {}; --namespace for the methods in WhammyButton

W_Options={
	["Mute"]=nil, --ailment muting
	["ShowDropdown"]=nil,
	["Temp4"]=nil,
	["Temp5"]=nil,
	["ThreatNotify"]=nil,
	["Movable"]=nil,
	["ShowSummary"]=nil,
	["ShowWhammy"]=nil, --is whammy visible
	["ButtonSize"]=nil,
	["Temp2"]=nil,
	["RangeCheck"]=nil,
	["Health1"]=nil,
	["Health2"]=nil,
	["Angle"]=nil,
	["Actions"]={},
	["Actions2"]={}
}
--/run print((select(4, GetBuildInfo())))

--local gInParty, gInRaid --in a raid or in a party
local gRaidList = {} --a list of raid1, raid2, raid3, used instead of concatenating raid..1  raid..2 etc for speed
local gPartyList = {} --
for i = 1,4 do
	gPartyList = format("party%d", i)
end
for i = 1,40 do
	gRaidList = format("raid%d", i)
end

function Whammy_OnLoad( self )
	-- Register the game events neccesary for our functionality
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("VARIABLES_LOADED");
	
	--self:RegisterEvent("RAID_ROSTER_UPDATE");
	--self:RegisterEvent("PARTY_MEMBERS_CHANGED");

	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MANA");
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("PLAYER_REGEN_DISABLED"); --use this to know if we enter or leave
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_ALIVE");

	-- note that the Talent change event is registered after the player login event
	
	--theFrame:RegisterEvent("RAID_TARGET_UPDATE");
	--theFrame:RegisterEvent("UNIT_THREAT_LIST_UPDATE"); --called when a targeted mobs threat list changes
	--theFrame:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE"); --called when a unit (party/raid) moves past another unit
	
	-- set up the slash commands
	SlashCmdList["WHAMMY"] = Whammy_Slash_Command_Handler;
	SLASH_WHAMMY1 = "/whammy";
	SLASH_WHAMMY2 = "/wham";

	WhammyVers = GetAddOnMetadata("Whammy", "Version");  --will be 0.0.7 xx/xx/xx

	self.TimeSinceLastUpdate = 0; --used for range checking
	W.FrameInitWhammyFrame( self ); --create the main frame
	W.FrameCreateUnitButtons( self ); --create the buttons

	W.FrameCreateTestButtons( self ); --the buttons for testing
	
	--W_Test_Button:Show();
	--W_Test_Button2:Show();
end
-- 
-- --------
-- --------  EVENT HANDLERS --------
-- --------

function W.Event_AddOnLoaded()
	--W.ShowMessage("W.Event_AddOnLoaded")

	--W.ShowMessage("W.Event_AddOnLoaded Complete")
end

function W.Event_VariablesLoaded()
	local f = WhammyFrame;
	
	W.Init_Variables(); --if any vars are nil, set to default valuess
	
	if ( W_Options.ShowWhammy ) then --if the last time the player played, Whammy was visible, keep it visible'
		if ( f ) then	
			f:Show();
		end
	end
	
	--f:SetMovable( W_Options.Movable ); --either lock the frame in place or make it movable
	
	WhammyButton_Move(); --move the Whammy Minimap icon into position
	WC.Init_Controls(); --now load the values into the config window.
end

-- This function is called from the main Whammy frame every W_UpdateInterval to check for range on the buttons. On Update is set in WhammyConfig
function W.Event_FrameOnUpdate(theFrame, elapsed)
	theFrame.TimeSinceLastUpdate = theFrame.TimeSinceLastUpdate + elapsed;
	
	if ( theFrame.TimeSinceLastUpdate > W_kUpdateInterval ) then
		W.UnitButtonRangeCheck( theFrame );
		theFrame.TimeSinceLastUpdate = 0;
	end
	
	-- this is tighter code execution and may be used in future updates if necessary
	-- while (self.TimeSinceLastUpdate > W_kUpdateInterval) do
	-- 	W.ShowMessage("Whammy Update message"..elapsed)
	--     self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - W_kUpdateInterval;
	-- end
end

function W.Event_PlayerLogin()
	--W.ShowMessage("    Event_PlayerLogin")
	local f = WhammyFrame;

	W.FrameDrawUnitButtons( f ); --draw the buttons in their proper positions
	WC.DropSlot_Init(); --set up the drop slots, which will in turn setup the unit buttons. Needs to be called here since we poll the spellbook for spell info.

	if ( not IsInGroup() ) then --if the player is not in a group set it as so, otherwise the group events will fire and set to group
		W.SetGroupAsPlayer();
	end

	-- local playerGroupStatus = W.GetPlayerGroupStatus();
	-- 
	-- if ( playerGroupStatus == kPlayerAlone ) then
	-- 	W.SetGroupAsPlayer();
	-- end
	
	--at this point, talent information is available so register that event
	f:RegisterEvent("PLAYER_TALENT_UPDATE");
end



-- function W.Event_PartyMembersChanged()
-- 	--W.ShowMessage("    Event_PartyMembersChanged")
-- 	local f = WhammyFrame;
-- 
-- 	f.unitIDArray = {} --reset this as it's about to be re-loaded
-- 	
-- 	W.UnitButtonSetName(1, "player"); --set button1 to player
-- 
-- 	for i = 1,4 do --player is in pos #1, 1-4 are the other 4 members
-- 		local unitID = "party"..i;
-- 		local buttonNum = i+1; --button 1 is player, button2 is party1, button3 is party2 etc
-- 		if ( UnitExists(unitID) ) then
-- 			W.UnitButtonSetName( buttonNum, unitID ); --party1 goes to button2, party2 goes to button3 etc. Also sets up the health amount for the unit
-- 		else
-- 			W.UnitButtonReset(buttonNum);
-- 		end
-- 	end
-- end

function W.Event_GroupRosterUpdate()
	--W.ShowMessage("    Event_GroupRosterUpdate");
	
	W.UnitButtonResetAll(); --clear out all the unit buttons as the group setup changed and we are re-loading them
	local f = WhammyFrame;
	f.unitIDArray = {} --reset this as it's about to be re-loaded
	
	local playerGroupStatus = W.GetPlayerGroupStatus();
	
	if ( playerGroupStatus == kPlayerAlone ) then
		W.SetGroupAsPlayer();
	elseif ( playerGroupStatus == kPlayerInParty ) then
		W.SetGroupAsParty();
	elseif ( playerGroupStatus == kPlayerInRaid ) then
		W.SetGroupAsRaid();
	else
		--should never get here
	end
end

function W.SetGroupAsPlayer()
	--W.ShowMessage("    player is alone");
	W.UnitButtonSetName(1, "player"); --set button1 to player
	WC.RangeCheckEnable( false ); --no point in range checking myself
end

function W.SetGroupAsParty()
	--W.ShowMessage("    SetGroupAsParty")
	
	W.UnitButtonSetName(1, "player"); --set button1 to player
			
	for i = 1,4 do --player is in pos #1, 1-4 are the other 4 members
		local unitID = "party"..i;
		local buttonNum = i+1; --button 1 is player, button2 is party1, button3 is party2 etc
		if ( UnitExists(unitID) ) then
			W.UnitButtonSetName( buttonNum, unitID ); --party1 goes to button2, party2 goes to button3 etc. Also sets up the health amount for the unit
		else
			W.UnitButtonReset(buttonNum);
		end
	end
	
	if ( W_Options.RangeCheck == true ) then
		WC.RangeCheckEnable( true ); --we are in a party, start range checking if that option is enabled
	end
end

function W.SetGroupAsRaid()
	--W.ShowMessage("    SetGroupAsRaid");
	
	local f = WhammyFrame;
	local unitCount, buttonNum, id, unitID;
	local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML;
	local subGroupUnitCount = {0,0,0,0,0,0,0,0}; --keep track of how many raid members are in each raid group. Starts at 0 (5 per group)
	
	local numGroupMembers = GetNumGroupMembers();
	--W.ShowMessage ("numberGroupMembers "..numGroupMembers );
	
	for id = 1,MAX_RAID_MEMBERS do --this will run from 1 to whatever the maximum # of raid members is
		if ( id <= numGroupMembers ) then --this will only examine raid members 1...n, where n=number of actual raid members
			unitID = "raid"..id
			name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(id);	--get their name and subgroup
			subGroupUnitCount[ subgroup ] = subGroupUnitCount[ subgroup ] + 1; --keep track of how many units are in each subgroup
			unitCount = subGroupUnitCount[ subgroup ]; --get the next unit position based on the unit count of the subgroup
			buttonNum = ( ( (subgroup-1) * 5) + unitCount ) -- first subgroup, first position will be 1, 8th subgroup, first position will be 36
			W.UnitButtonSetName( buttonNum, unitID ); --sets the name and background color
		end
	end
	
	if ( W_Options.RangeCheck == true ) then
		WC.RangeCheckEnable( true ); --we are in a raid, start range checking if that option is enabled
	end
end

function W.Event_PlayerTargetChanged( someArg )
	--W.ShowMessage("Event_PlayerTargetChanged")
	W.UnitButtonClearThreat();
end

function W.Event_PlayerTalentUpdate()
	--W.ShowMessage("player talent update occured");
	WC.DropSlot_Init(); --set up the new talents
end

function W.Event_UnitAura( unitID )
	local f = WhammyFrame;
	local isRemovable = 1;
	local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3;
	
	W.UnitButtonSetAura( unitID, nil ); --clear the aura for this unitID button
	for debuffNum = 1,40 do
		name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3 = UnitDebuff(unitID, debuffNum, isRemovable)
		if ( dispelType ) then -- if theres a debuff:  Magic Disease Poison Curse
			local alreadyHasAilment = W.UnitButtonGetAura( unitID, dispelType); --see if this unit is already showing the ailment
			if ( alreadyHasAilment == false ) then
				W.UnitButtonSetAura( unitID, dispelType ); -- indicate it on the unit button
				if ( W_Options.Mute == false ) then -- if we are not muted, play a sound
					--W.ShowMessage("UnitID = "..unitID.."  Aura = "..dispelType)
					PlaySoundFile("interface\\addons\\Whammy\\AilmentSound.mp3");
				end
			end
		end
	end
end

function W.Event_UnitHealth( unitID )
	--W.ShowMessage("Event_UnitHealth "..unitID)
	local healthFraction;
	local maxHealth = UnitHealthMax( unitID )
	local currentHealth = UnitHealth( unitID )
	
	if ( UnitIsConnected( unitID ) ) then
	
		if ( UnitIsDeadOrGhost( unitID ) or currentHealth < 1 ) then --unit dead
			healthFraction = 0;
		else
			healthFraction = currentHealth / maxHealth; --the % of how much health they have
			
			if (healthFraction < 0) then healthFraction = 0; end;
			if (healthFraction > 1) then healthFraction = 1; end;
		
			local healthAmount = healthFraction * 100;
			if ( W_Options.Health2 ~= "Off" and healthAmount < tonumber(W_Options.Health2) ) then
				W.UnitButtonHealthAlertSet( unitID, 2, true)
			elseif ( W_Options.Health1 ~= "Off" and healthAmount < tonumber(W_Options.Health1) ) then
				W.UnitButtonHealthAlertSet( unitID, 1, true)
			else
				W.UnitButtonHealthAlertSet( unitID, nil, nil)
			end
		end
	
		W.UnitButtonSetHealth( unitID, healthFraction ); --will adjust the texture of the button to indicate the amount of health
	-- else
	-- 	W.ShowMessage("disconnected")
	-- 	W.UnitButtonSetDisconnected( unitID )
	end
end

function W.Event_UnitThreatListUpdate( mobID ) --the Mob is the mob whose threat list changed or we just went into combat
	--W.ShowMessage("Event_UnitThreatListUpdate "..mobID)
	--one way to check the mobID is to see who it's target it
	-- if ( UnitExists(mobID) ) then
	-- 
	-- 	if ( not UnitIsPlayer(mobID) and UnitCanAttack( "player", mobID) and UnitHealth( mobID) > 0 and not UnitPlayerControlled(mobID) ) then
	-- 		local mobTarget = mobID.."target"; --this is who has aggro on the mob
	-- 		local mobTargetGUID = UnitGUID(mobTarget); --the get tanks GUID
	-- 		if ( mobTargetGUID ) then
	-- 			--guidNameLookup[mobTargetGUID] = UnitName(mobTarget); --store it
	-- 			local tankName = UnitName( mobTarget );
	-- 			W.ShowMessage("Threat List Update *** "..tankName.." *** HAS AGGRO");
	-- 		end
	-- 	
	-- 	end
	-- end

	--another way is to to check the raid to see who is firmly tanking the mob
	-- local aList;
	-- 
	-- if ( inParty or inRaid ) then
	-- 	if ( inRaid ) then
	-- 		aList = gRaidList; --array of raid1, raid2 etc
	-- 		currentPartySize = GetNumGroupMembers();
	-- 	else
	-- 		aList = gPartyList; --array of party1, party2 etc
	-- 		currentPartySize = GetNumGroupMembers();
	-- 	end
	-- 	
	-- 	for i = 1, currentPartySize do
	-- 		local unitID = aList[i];
	-- 		status = UnitThreatSituation( unitID, mobID)
	-- 		if ( status == 3 ) then --this unit has 100% threat
	-- 			local unitName = UnitName( unitID )
	-- 			W.ShowMessage("*** "..unitName.." *** HAS AGGRO")
	-- 		end
	-- 	end
	-- 
	-- end
end

function W.Event_UnitThreatSituationUpdate( unitID ) --the unit is the unit in the party or raid who changed threat position
	--unitID could be player, party1, raid1 etc... need to get the name
	local mobID = "playertarget"; --who is my target
	if ( unitID ) then
		if ( not UnitIsPlayer(mobID) and UnitCanAttack( "player", mobID) and UnitHealth( mobID) > 0 and not UnitPlayerControlled(mobID) ) then
		
			local mobName = UnitName( mobID );
			local uName = UnitName( unitID );
			if mobName and uName then --if we have a valid mob targeted and a valid unitID was passed in then check threat
				local status = UnitThreatSituation( unitID, mobID ); --if status = 3, the unitID has aggro
				--W.ShowMessage("Event_UnitThreatSituationUpdate *** "..unitID.." *** has aggro on "..mobName);
				local msg = "*** "..uName.." *** has aggro on "..mobName
				UIErrorsFrame:AddMessage( msg, 1.0, 0.0, 0.0, 53, 5);
				local b = W.GetButtonFromUnitID( unitID );
				if (b) then --if there is a button for this unit (i.e. they are in our raid or party)
					W.UnitButtonSetThreat( b ); --set the threat indicator active, and turn it off on others
				end
			end
		 end
		
	end	
	-- local playerGUID = UnitGUID( unitID ); --the global unit number of this unit
	-- if playerGUID then
	-- 	W_gGUIDNameTable[playerGUID] = UnitName( unitID ); --store the unitname in a global dictionary according to their GUID
	-- 	_, w_gGUIDClassTable[playerGUID] = UnitClass( unitID );
	-- end
end

-- called when we enter or leave combat. Enable or disable the drop downs depending on combat status
function W.Event_InCombat( f, isInCombat ) --isInCombat = true when entering combat, and false when leaving
	
	if ( isInCombat ) then --when in combat disable the growbox so we don't taint the frame and get an error, register the event to detect when we leave combat
		--W.ShowMessage("in combat")
		f:RegisterEvent("PLAYER_REGEN_ENABLED")
		f:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
		f:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		f:RegisterEvent("PLAYER_TARGET_CHANGED")
		
		f.sizer:SetScript("OnMouseDown", nil)
		f.sizer:SetScript("OnMouseUp",   nil)
	else --we have left combat, unlock frames, perform updates etc
		--W.ShowMessage("left combat")
		f:UnregisterEvent("PLAYER_REGEN_ENABLED") --already got the event so forget about it
		f:UnregisterEvent("UNIT_THREAT_LIST_UPDATE")
		f:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		f:UnregisterEvent("PLAYER_TARGET_CHANGED")
		
		W.Event_GroupRosterUpdate();
		
		f.sizer:SetScript("OnMouseDown", function( self ) W.FrameGrowBoxOnMouseDown( self ); end)
		f.sizer:SetScript("OnMouseUp",   function( self ) W.FrameGrowBoxOnMouseUp( self ); end)
		W.UnitButtonClearThreat();
		W.FrameDrawUnitButtons( f );
	end
	
	W.Event_InCombat_ShowBorder( isInCombat );

	--when in combat, lock down the DropBoxes so we don't try to change them, or unlock them when we leave combat
	for dropSlotNum = 1,20 do
		local ds = getglobal( "WC_DropSlot"..dropSlotNum );
		if ( isInCombat ) then
			ds.texture = ds:CreateTexture(ds:GetName().."_Texture", "ARTWORK")
			ds.texture:SetTexture(.5, .5, .5, .5)
			ds.texture:SetAllPoints(ds)
			ds:Disable()
		else
			if (ds.texture) then
				ds.texture:SetTexture(nil)
			end
			ds:Enable()
		end
	end
end

function W.Event_InCombat_ShowBorder( isInCombat )
	local f = WhammyFrame;

	if ( isInCombat ) then --show a faint red border around Whammy when in combat
		--W.ShowMessage( "showing border")
		f.topborder:Show();
		f.rightborder:Show();
		f.bottomborder:Show();
		f.leftborder:Show();
	else --hide the border
		--W.ShowMessage( "hiding border")
		f.topborder:Hide();
		f.rightborder:Hide();
		f.bottomborder:Hide();
		f.leftborder:Hide();
	end

end
-- --
-- --------  END EVENT HANDLERS --------
-- --
-- 
-- called once the variables are loaded - if they are nil, set to defaults
function W.Init_Variables()
	--W.ShowMessage("W.Init_Variables")
	if ( W_Options == nil ) then W_Options = {}; end
	if ( W_Options.Mute == nil ) then W_Options.Mute = false; end
	if ( W_Options.ShowDropdown == nil ) then W_Options.ShowDropdown = false; end
	if ( W_Options.Temp4 == nil ) then W_Options.Temp4 = false; end
	if ( W_Options.Temp5 == nil ) then W_Options.Temp5 = false; end
	if ( W_Options.ThreatNotify == nil ) then W_Options.ThreatNotify = true; end
	if ( W_Options.Movable == nil ) then W_Options.Movable = true; end
	if ( W_Options.ShowSummary == nil ) then W_Options.ShowSummary = true; end
	if ( W_Options.ShowWhammy == nil ) then W_Options.ShowWhammy = true; end
	if ( W_Options.ButtonSize == nil ) then W_Options.ButtonSize = W_kButtonSizeStandard; end
	if ( W_Options.Temp2 == nil ) then W_Options.Temp2 = false; end
	if ( W_Options.RangeCheck == nil ) then W_Options.RangeCheck = true; end
	if ( W_Options.Health1 == nil ) then W_Options.Health1 = "Off"; end
	if ( W_Options.Health2 == nil ) then W_Options.Health2 = "Off"; end
	if ( W_Options.Angle == nil ) then W_Options.Angle = 0; end
	if ( W_Options.Actions == nil ) then W_Options.Actions = {}; end --each row is a table with (item,spell or macro), ID, and either itemLink or spell/pet
	if ( W_Options.Actions2 == nil ) then W_Options.Actions2 = {}; end --the actions for secondary spec
end

function W.UnitButtonHealthAlertSet( unitID, whichAlert, isSet)
	local b = W.GetButtonFromUnitID( unitID )
	local red, green, blue, alpha;
	if b then
		if ( isSet ) then
			if ( whichAlert == 1) then --yellow
				red=1; green=1; blue=0; alpha=1
			elseif ( whichAlert == 2 ) then --red
				red=1; green=0; blue=0; alpha=1
			end
			b.healthAlertLeftTexture:SetTexture( red, green, blue, alpha);
			b.healthAlertTopTexture:SetTexture( red, green, blue, alpha);
			b.healthAlertRightTexture:SetTexture( red, green, blue, alpha);
			b.healthAlertBottomTexture:SetTexture( red, green, blue, alpha);
		else
			b.healthAlertLeftTexture:SetTexture( nil );
			b.healthAlertTopTexture:SetTexture( nil );
			b.healthAlertRightTexture:SetTexture( nil );
			b.healthAlertBottomTexture:SetTexture( nil );
		end
	end
end

function W.Test_Button_Click()
	--W.ShowMessage("Test Button #1 click")
	
	local b = WhammyFrame.buttonArray[1];
	
	local x = b:GetAttribute("type1");
	W.ShowMessage(x);
		
	local y = b:GetAttribute("spell1");
	W.ShowMessage(y);
	
	
	local theType = b:GetAttribute("shifttype1");
	W.ShowMessage(theType);
		
	local theAction = b:GetAttribute("shiftspell1");
	W.ShowMessage(theAction);

end

function W.Test_Button2_Click()
	--W.ShowMessage("Test Button #2 click")
	
end

function W.FrameInitWhammyFrame( f )

	f:SetWidth(400)
	f:SetHeight(200)
	f:SetPoint("CENTER", 0, 0)
	f:SetResizable(true)
	f:SetMaxResize( UIParent:GetWidth(), UIParent:GetHeight() - 50)
	f:EnableMouse(true)
	f:SetMovable(true)

	f.texture = f:CreateTexture("myframe_texture", "BACKGROUND")
	f.texture:SetTexture(.1,.1,.1,.4)
	f.texture:SetAllPoints(f)

	f.sizer = CreateFrame("Frame", "WhammyFrame_Sizer", f)
	f.sizer:SetWidth(16)
	f.sizer:SetHeight(16)
	f.sizer:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)
	f.sizer.texture = f.sizer:CreateTexture("myframe_sizer_texture", "ARTWORK")
	f.sizer.texture:SetAllPoints( f.sizer )
	f.sizer.texture:SetTexture("Interface\\AddOns\\Whammy\\Images\\FrameResizeIcon")
	f.sizer:EnableMouse();
	f.sizer:SetScript("OnMouseDown", function( self ) W.FrameGrowBoxOnMouseDown( self ); end)
	f.sizer:SetScript("OnMouseUp",   function( self ) W.FrameGrowBoxOnMouseUp( self ); end)
	
	f.buttonArray = {}; --keeps track of all the buttons in the frame
	f.unitIDArray = {}; --cross reference from the unit name stored in each button to the button the unit is stored in
	
	--this is the red border that's displays when you are in combat
	local red = 0.6;
	local alpha = 0.4;
	
	f.topborder = f:CreateTexture("top_border", "BACKGROUND");
	f.topborder:SetTexture(red, 0, 0, alpha)
	f.topborder:SetPoint("TOPLEFT", f, "TOPLEFT", -1, 1);
	f.topborder:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT", 0, 0);
	
	f.rightborder = f:CreateTexture("right_border", "BACKGROUND");
	f.rightborder:SetTexture(red, 0, 0, alpha)
	f.rightborder:SetPoint("TOPLEFT", f, "TOPRIGHT", 0, 1);
	f.rightborder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 1, -1);
	
	f.bottomborder = f:CreateTexture("bottom_border", "BACKGROUND");
	f.bottomborder:SetTexture(red, 0, 0, alpha)
	f.bottomborder:SetPoint("TOPLEFT", f, "BOTTOMLEFT", -1, -1);
	f.bottomborder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 1, -2);
	
	f.leftborder = f:CreateTexture("left_border", "BACKGROUND");
	f.leftborder:SetTexture(red, 0, 0, alpha)
	f.leftborder:SetPoint("TOPLEFT", f, "TOPLEFT", -2, 1);
	f.leftborder:SetPoint("BOTTOMRIGHT", f, "BOTTOMLEFT", -1, -2);
	
	W.Event_InCombat_ShowBorder( false );
	
	-- f.FrameBorder = CreateFrame("Frame", nil, f);
	-- f.FrameBorder:SetPoint("TOPLEFT", f, "TOPLEFT", -1, 1);
	-- f.FrameBorder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 1, -1);
	-- --f.FrameBorder:SetFrameStrata("BACKGROUND");
	-- f.FrameBorder.texture = f.FrameBorder:CreateTexture("border_texture", "BACKGROUND");
	-- f.FrameBorder.texture:SetTexture(.4, 0, 0, .4);
	-- f.FrameBorder.texture:SetAllPoints(f.FrameBorder);
	
	
	-- f.FrameBorder = CreateFrame("Frame", nil, f);
	-- f.FrameBorder:SetPoint("TOPLEFT", f, "TOPLEFT", -4, 4);
	-- f.FrameBorder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 4, -3);
	-- f.FrameBorder:SetFrameStrata("BACKGROUND");
	-- f.FrameBorder:SetBackdrop {
	-- 		edgeFile = "Interface/Tooltips/UI-Tooltip-Border", edgeSize = 10,
	-- 		insets = {left = 0, right = 0, top = 0, bottom = 0}
	-- 	}
	-- f.FrameBorder:SetBackdropBorderColor(1, 0, 0, .6);
	
end

function Whammy_OnEvent( self, event, ... )
	local arg1 = ...;
	
	local theEvent = event;
	 -- if ( theEvent ) then
	 -- 	W.ShowMessage( "Whammy_OnEvent: "..theEvent )
	 -- end
	
	if ( event == "ADDON_LOADED" and arg1 == "Whammy") then -- the event occurs after the saved vars are loaded
		--W.ShowMessage("Version "..WhammyVers.."   /Whammy to show/hide Whammy"); --" V0.21a 1/6/2007 Loaded");
		self:UnregisterEvent("ADDON_LOADED");
		W.Event_AddOnLoaded(); --set up button frames, etc
		
	elseif ( event == "VARIABLES_LOADED" ) then --this event occurs after we load in any saved variables.
		W.Event_VariablesLoaded();
		
	elseif ( event =="PLAYER_LOGIN") then
		W.Event_PlayerLogin()
		
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then --we have entered combat
		W.Event_InCombat( self, true )
		
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then --we have left combat, so do whatever updates we couldn't do during combat
		W.Event_InCombat( self, false )
		
	elseif ( event == "UNIT_THREAT_LIST_UPDATE") then
		W.Event_UnitThreatListUpdate( arg1 ); --pass the mob whose threat list changed
		
	elseif ( event == "UNIT_THREAT_SITUATION_UPDATE" and W_Options.ThreatNotify ) then
		W.Event_UnitThreatSituationUpdate( arg1 ); --pass the unit whose threat has moved on the mobs list

	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		W.Event_PlayerTargetChanged( arg1 ); --arg1 doesn't matter
		
	elseif ( event == "GROUP_ROSTER_UPDATE") then -- the group has changed, players added or disbanded
		if ( InCombatLockdown() ) then
			self:RegisterEvent("PLAYER_REGEN_ENABLED")
		else
			W.Event_GroupRosterUpdate();
		end

	elseif ( event == "UNIT_AURA" ) then --handle curses or raid or party members, ignore pets
		if ( UnitIsUnit( arg1, "player") or UnitInParty(arg1) or UnitInRaid(arg1) ) then --only if unit is the player or another player in the party or raid
			W.Event_UnitAura(arg1); --arg1 us the unitID
		end
		
	elseif ( event == "UNIT_HEALTH" ) then
		W.Event_UnitHealth(arg1); --arg1 is the unitID

	elseif ( event == "UNIT_MANA" and W.IsUnitIDPlayerPartyRaid( arg1 ) ) then
		WS.Event_UnitMana(); --arg1 is player, party1-4 or raid 1-40 mana for player, party or raid, this function is in WhammySummary.lua

	elseif ( event == "PLAYER_TALENT_UPDATE" ) then
		W.Event_PlayerTalentUpdate();
	end
end

	--elseif ( event == "UNIT_HEALTH" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then

function W.FrameCreateUnitButtons( theFrame )
	--W.ShowMessage("W.FrameCreateUnitButtons")
	local strata = "ARTWORK"
	
	for i = 1,W_kMaxUnitButtons do
		local bName = "W_Button"..i;
		local b = CreateFrame("Button", bName, theFrame, "WhammyUnitButtonTemplate");		--the button frame
		tinsert(theFrame.buttonArray, b)
		b:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp", "Button4Up", "Button5Up"); -- the buttons need to see the mouse clicks
		b:SetWidth(W_kButtonWidth);
		b:SetHeight(W_kButtonHeight);
		-- the texture which adjusts depending on health
		b.texture = b:CreateTexture(bName.."_Texture", "BACKGROUND");
		b.texture:SetAllPoints(b);
		b.texture:SetTexture( 1, 1, 1, .4 );
		b:Disable();
		
		-- the unit name
		b.unitName = b:CreateFontString( bName.."_UnitName", strata, W_kFontNameOthers )
		b.unitName:SetPoint("CENTER", b, "CENTER", 0, 0);

		--the alert indicator for low health warnings 1 and 2
		b.healthAlertLeftTexture = b:CreateTexture( bName.."HealthAlert_Left_Texture", strata);
		b.healthAlertLeftTexture:SetPoint( "TOPLEFT", b, "TOPLEFT", 0, 0 );
		b.healthAlertLeftTexture:SetPoint( "BOTTOMRIGHT", b, "BOTTOMLEFT", 2, 0);
		b.healthAlertTopTexture = b:CreateTexture( bName.."HealthAlert_Top_Texture", strata);
		b.healthAlertTopTexture:SetPoint( "TOPLEFT", b, "TOPLEFT", 0, 0 );
		b.healthAlertTopTexture:SetPoint( "BOTTOMRIGHT", b, "TOPRIGHT", 0, -2);
		b.healthAlertRightTexture = b:CreateTexture( bName.."HealthAlert_Right_Texture", strata);
		b.healthAlertRightTexture:SetPoint( "TOPRIGHT", b, "TOPRIGHT", 0, 0 );
		b.healthAlertRightTexture:SetPoint( "BOTTOMLEFT", b, "BOTTOMRIGHT", -2, 0);
		b.healthAlertBottomTexture = b:CreateTexture( bName.."HealthAlert_Bottom_Texture", strata);
		b.healthAlertBottomTexture:SetPoint( "BOTTOMLEFT", b, "BOTTOMLEFT", 0, 0 );
		b.healthAlertBottomTexture:SetPoint( "TOPRIGHT", b, "BOTTOMRIGHT", 0, 2);
			
		--When the unit gets a curse (or other ailment) the top lights up in yellow. Create those areas
		b.curseTexture = b:CreateTexture(bName.."_CurseTop", strata)
		b.curseTexture:SetPoint( "TOPLEFT", b, "TOPLEFT", 0, 0 );
		b.curseTexture:SetPoint( "BOTTOMRIGHT", b, "TOPRIGHT", 0, -4);
		
		--When in a party or raid, and a unit gets aggro, display something on the button to indicate they have aggro
		b.threatTexture = b:CreateTexture( bName.."_Threat", strata)
		b.threatTexture:SetPoint( "TOPRIGHT", b, "RIGHT", 0, 2 );
		b.threatTexture:SetPoint( "BOTTOMLEFT", b, "RIGHT", -8, -2);
		
		--When the unit gets a curse( or other ailment ), there are letters on the curse bar that indicate what it is: M)agic C)urse D)isease P)oison
		local y = 4;
		b.magicText = b:CreateFontString( (bName.."_Curse_TextM"), "OVERLAY", "GameFontNormalSmall" )
		b.magicText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", -27, y)
		b.magicText:SetTextColor( 1, 0, 0 );
		b.curseText = b:CreateFontString( (bName.."_Curse_TextC"), "OVERLAY", "GameFontNormalSmall" )
		b.curseText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", -17, y)
		b.curseText:SetTextColor( 1, 0, 0 );
		b.diseaseText = b:CreateFontString( (bName.."_Curse_TextD"), "OVERLAY", "GameFontNormalSmall" )
		b.diseaseText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", -9, y)
		b.diseaseText:SetTextColor( 1, 0, 0 );
		b.poisonText = b:CreateFontString( (bName.."_Curse_TextP"), "OVERLAY", "GameFontNormalSmall" )
		b.poisonText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", 1, y)
		b.poisonText:SetTextColor( 1, 0, 0 );
		-- this is the little Rng that appears on each unit button when the unit is out of range
		b.rangeText = b:CreateFontString( bName.."_RangeText", "OVERLAY", "GameFontNormalSmall");
		b.rangeText:SetPoint("BOTTOMRIGHT", b, "BOTTOMRIGHT", 1, -2);
		b.rangeText:SetText("");
		b.rangeText:SetTextHeight(10);
		b.rangeText:SetTextColor(1, .5, 1)
	end
	--W.ShowMessage("W.FrameCreateUnitButtons complete")
end

function W.FrameGrowBoxOnMouseDown( theGrowBox )
	theGrowBox:GetParent():StartSizing("BOTTOMRIGHT")
end

function W.FrameGrowBoxOnMouseUp( theGrowBox )
	local f = theGrowBox:GetParent();
	f:StopMovingOrSizing()
	W.FrameDrawUnitButtons(f);
end

function Whammy_MouseDown( self, ... )
	local theMouseButton = ...;
	
	if ( theMouseButton == "LeftButton" ) then --if we clicked the left button
		if ( W_Options.Movable ) then --and moving is enabled
			self:StartMoving(); --then move the frame
		end
	elseif ( theMouseButton == "RightButton" and ( IsLeftShiftKeyDown() or IsRightShiftKeyDown() )) then --if right click + shift, let the user lock/unlock the frame
		--present a popup list to lock or unlock the frame
		W.FrameMenuIsFrameMovable();
	end
end

function Whammy_MouseUp( self )
	self:StopMovingOrSizing()
end

function W.FrameMenuIsFrameMovable() --display a popup to allow the Whammy Frame to be set to Movable or not
	local menuFrame = CreateFrame("Frame", "ExampleMenuFrame", UIParent, "UIDropDownMenuTemplate")

	local function onOption1()
		W_Options.Movable = true;
		if (WhammyConfigFrame) then
			WhammyConfigFrame.isFrameMovableButton:SetChecked( W_Options.Movable );
		end
	end
	local function onOption2()
		W_Options.Movable = false;
		if (WhammyConfigFrame) then
			WhammyConfigFrame.isFrameMovableButton:SetChecked( W_Options.Movable );
		end
	end

	local menuList = {
	  { text = "Movable", func = onOption1 },
	  { text = "Locked", func = onOption2 }
	}

	EasyMenu(menuList, menuFrame, "cursor", 0 , 0, "MENU");
end



function W.FrameDrawUnitButtons( theFrame )
	--W.ShowMessage("W.FrameDrawUnitButtons")
	local bHeight = W_kButtonHeight + W_kGutter
	local bWidth, w;
	local frameWidth = theFrame:GetWidth();
	local frameHeight = theFrame:GetHeight();
	local rightLimit = frameWidth - W_kFrameGutterRight; --we start drawing at 0, not bWidth so we need to compensate for that
	local x = 0;
	local y = -10;
	local col = 0;

	if ( W_Options.ButtonSize == W_kButtonSizeStandard ) then
		w = W_kButtonWidth
		bWidth = w + W_kGutter;
	else
		w = W_kButtonWidthSmall
		bWidth = w + W_kGutter;
	end
	--sets the min size of the whammyframe depending on the size of the buttons.
	theFrame:SetMinResize( w + W_kGutter + W_kFrameGutterRight, W_kButtonHeight+3+W_kFrameGutterTop)

	--draw the buttons
	for buttonNum = 1,W_kMaxUnitButtons do
		local b = theFrame.buttonArray[buttonNum];
		b:SetWidth( w );
		W.UnitButtonSetNameWidth( b );
		
		x = col * bWidth;
		
		if ( x > rightLimit or (x+bWidth) > rightLimit ) then --if this button would go past the right side of the frame, drop it down a line
			x = 0;
			col = 0;
			y = y - bHeight
		end
		
		if ( -y > frameHeight or (-y+bHeight) > frameHeight ) then
			b:Hide()
		else
			b:Show()
		end
		
		b:ClearAllPoints()
		b:SetPoint("TOPLEFT", theFrame, "TOPLEFT", x, y)
		col = col + 1;
	end
end

function W.FrameCreateTestButtons( theFrame )
	local testButton1 = CreateFrame("Button", "W_Test_Button", theFrame, "UIPanelButtonTemplate")
	testButton1:SetWidth(80);
	testButton1:SetHeight(22);
	testButton1:SetPoint("TOPLEFT", theFrame, "BOTTOMLEFT", 5, 5);
	testButton1:SetScript("OnClick", function() W.Test_Button_Click(); end);
	testButton1:SetText("Test1");
	testButton1:Hide()

	local testButton2 = CreateFrame("Button", "W_Test_Button2", theFrame, "UIPanelButtonTemplate")
	testButton2:SetWidth(80);
	testButton2:SetHeight(22);
	testButton2:SetPoint("RIGHT", theFrame, "LEFT", 5, 5)
	testButton2:SetScript("OnClick", function() W.Test_Button2_Click(); end);
	testButton2:SetText("Test2");
	testButton2:Hide()
end

function W.TestBoolean( someB )
	if (someB == 1) then
		return "true"
	elseif (someB == 0) then
		return "false"
	else
		return "nil"
	end
end

function W.UnitButtonReset( i )
	local f = WhammyFrame
	local b = f.buttonArray[i];	

	--reset the buttons and cancel watching for clicks.
	b:SetScript("OnMouseDown", nil ); --thebutton and the mousebutton
	b:SetScript("OnMouseUp", nil);

	b.unitName:SetText(nil)

	b.texture:SetTexture( 1, 1, 1, .4 ); --sets the background to default
	b.texture:SetAllPoints( b ); --sets the texture to full size (health)

	b.magicText:SetText(nil); --clear out any aura's
	b.curseText:SetText(nil);
	b.diseaseText:SetText(nil);
	b.poisonText:SetText(nil);

	b.curseTexture:SetTexture( nil ); --clear the curse bar

	b.rangeText:SetText(nil); --clear out the range text

	b.unitName:SetFontObject(W_kFontNameOthers); --reset the font

	b.healthAlertLeftTexture:SetTexture( nil ); --reset the healh alert indicators
	b.healthAlertTopTexture:SetTexture( nil );
	b.healthAlertRightTexture:SetTexture( nil );
	b.healthAlertBottomTexture:SetTexture( nil );

	local unitID = b:GetAttribute("unit"); --get the unitID if there was one
	if (unitID) then --if the array that links unitID's to buttons had an entry for this, then remove it
		f.unitIDArray[unitID] = nil; --removes the unitID entry from the dictionary
	end
	b:SetAttribute("type", "stop"); --set the function of the button to nothing
	b:SetAttribute("unit", nil)

	b:Disable(); --disable the button

end

function W.UnitButtonResetAll()
	for i = 1,W_kMaxUnitButtons do
		W.UnitButtonReset(i)
	end
end

function W.UnitButtonSetAura( unitID, theAura) --when theAura=nil, then clear the debuff spot
	local b = W.GetButtonFromUnitID( unitID )

	if (b) then --there could be a time when, you are in combat, and added someone to the party/raid that b could be nil
		if ( theAura ) then --there is a debuff, figure out what it is and display the text
			--W.ShowMessage("Setting "..theAura);
			if (theAura == "Magic") then --if it was magic then set the magic text otherwise clear it
				b.magicText:SetText(W_kMagicText);
			elseif (theAura == "Curse") then
				b.curseText:SetText(W_kCurseText);
			elseif (theAura == "Disease") then
				b.diseaseText:SetText(W_kDiseaseText);
			elseif (theAura == "Poison") then
				b.poisonText:SetText(W_kPoisonText);
			end
			
			local result = b.curseTexture:SetTexture( .97, 1, .38, 1)--yellow bar across the top of the button
			--don't need to worry about result at this time
		else
			b.magicText:SetText(nil); --if there are no debuffs, clear them all
			b.curseText:SetText(nil);
			b.diseaseText:SetText(nil);
			b.poisonText:SetText(nil);
			
			b.curseTexture:SetTexture( nil )
		end	
	end
end

function W.UnitButtonGetAura( unitID, theAura)  -- determine if this unit already has the specified ailment, return true if so
	local b = W.GetButtonFromUnitID( unitID )
	local theText;
	
	if (b) then
		if (theAura == "Magic") then
			theText = b.magicText:GetText();
		elseif (theAura == "Curse") then
			theText = b.curseText:GetText();
		elseif (theAura == "Disease") then
			theText = b.diseaseText:GetText();
		elseif (theAura == "Poison") then
			theText = b.poisonText:GetText();
		end
	end
	
	if ( theText ) then
		return true;
	else
		return false;
	end
end

function W.UnitButtonSetHealth( unitID, healthAmountInDecimal )
	--W.ShowMessage("UnitButtonSetHealth");
	local buttonWidth;
	
	local b = W.GetButtonFromUnitID( unitID );--get the button to change based on the unitID whose health is changing
	if b then
		if ( healthAmountInDecimal > 0 ) then -- unit is alive so set up their health
			if ( W_Options.ButtonSize == W_kButtonSizeStandard ) then
				buttonWidth = W_kButtonWidth;
			else
				buttonWidth = W_kButtonWidthSmall;
			end
			W.UnitButtonSetClassColorAndStatus( b, unitID ); --check the range and set the colors
			local delta = buttonWidth - (buttonWidth * healthAmountInDecimal); --if w=100 and health=100%, then this would be 100-(100*1) = 0
			b.texture:SetPoint("TOPLEFT", b ,"TOPLEFT", 0, 0);
			b.texture:SetPoint("BOTTOMRIGHT", b, "BOTTOMRIGHT", -delta, 0);
		else --they are dead, dim the texture
			W.UnitButtonSetAura( unitID, nil ) --reset the aura's and curse bar
			W.UnitButtonSetClassColorAndStatus( b, unitID ); --check the range and set the colors
		end
	-- else
	-- 	W.ShowMessage("unitButtonSetHealth, b was nil")
	end
end

function W.UnitButtonSetName(whichButtonNum, unitID)
	--W.ShowMessage("W.UnitButtonSetName:  buttonNum = "..whichButtonNum.."  unitID = "..unitID);
	--whichButtonNum is the button number 1-40 and the unitID will be raid1, raid2 etc
	local f = WhammyFrame;
	local b = f.buttonArray[whichButtonNum];
	
	--since it has a unit, accept clicks and handle them
	b:SetScript("OnMouseDown", function(self, arg1) W.UnitButtonMouseDown(self, arg1); end); --thebutton and the mousebutton
	b:SetScript("OnMouseUp", function(self) W.UnitButtonMouseUp(self); end);
	
	f.unitIDArray[unitID] = b; --associate a unitID table with the button so we can look up buttons via a unitID
	--W.ShowMessage("UnitButtonSetName unitID.."..unitID.." button ="..b:GetName() )
	b.unitName:SetText( UnitName( unitID ) );
	W.UnitButtonSetNameWidth( b );
	b:SetAttribute("unit", unitID); --set the assignment of the button to the passed in unit
	W.UnitButtonSetClassColorAndStatus( b, unitID );
	W.Event_UnitHealth(unitID); --set the initial health for the unit
	
	if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
		b.unitName:SetFontObject(W_kFontNamePlayer)
	else --someone else in the party or raid
		b.unitName:SetFontObject(W_kFontNameOthers)
	end
	
	b:Enable();
end

function W.UnitButtonRangeCheck( f ) --f is whammyframe and this is called every x seconds to see if unit is out of range
	for buttonNum = 1,W_kMaxUnitButtons do
		local b = f.buttonArray[buttonNum];
		local unitID = b:GetAttribute("unit");
		--W.ShowMessage("UnitButtonRangeCheck")
		if (unitID) then
			W.UnitButtonSetClassColorAndStatus( b, unitID );
		end
	end
end

-- called when assigning a unit to a button. Determine what kind class of unit (warrior, mage etc) and color the backgroud to match
-- also called from the range checking to dim the button
function W.UnitButtonSetClassColorAndStatus(b, unitID)
	local class, classFileName = UnitClass(unitID); --get the type of class and a path to it's file, which I am not using at the moment
	local color = RAID_CLASS_COLORS[classFileName]; -- get the standard class colors in a color.r, color.g, color.b format
	local red, green, blue, alpha, d;
	local isPlayer = UnitIsUnit(unitID, "player");
	local inRange = UnitInRange(unitID);
	local isDead = UnitIsDead(unitID);
	local isGhost = UnitIsGhost(unitID);
	local isConnected = UnitIsConnected(unitID);
	
	if ( class ) then
		if ( isDead ) then --if dead clear to black, ghost clear to grey
			b.texture:SetTexture( 0, 0, 0, 1 ); --set to black when dead
			if ( inRange or isPlayer ) then --if in range or player, clear range text
				b.rangeText:SetText( "" );
			else --out or range or not player, show range text
				b.rangeText:SetText( W_kOutOfRangeText );
			end

		elseif (isGhost) then
			b.texture:SetTexture( 0.3, 0.3, 0.3, 0.9 );
			if ( inRange or isPlayer ) then --if in range or player, clear range text
				b.rangeText:SetText( "" );
			else --out or range or not player, show range text
				b.rangeText:SetText( W_kOutOfRangeText );
			end

		elseif (isConnected == nil) then --disconnected, set to black and a disc label
			b.texture:SetTexture( 0, 0, 0, 1 )
			b.rangeText:SetText( W_kDisconnectedText );
		
		else --they are alive, the are either in range or out of range
			if (isPlayer) then --if the unit is this player, then full color and clear the disc or rng txt
				b.texture:SetTexture( color.r, color.g, color.b, 1 )
				b.rangeText:SetText( "" );
			else
				if ( inRange ) then
					d = 1;
					alpha = 1;
					b.rangeText:SetText( "" );
				else
					d = .8;
					alpha = .7
					b.rangeText:SetText( W_kOutOfRangeText );
				end
	
				red = color.r * d;
				green = color.g * d;
				blue = color.b * d;
				b.texture:SetTexture( red, green, blue, alpha )
			end
		end
	end
end

-- Message: Interface\AddOns\Whammy\Whammy.lua:1037: attempt to index local 'color' (a nil value)
-- Time: Sun Sep  2 15:37:50 2012
-- Count: 2
-- Stack: Interface\AddOns\Whammy\Whammy.lua:1037: in function `UnitButtonSetClassColorAndStatus'
-- Interface\AddOns\Whammy\Whammy.lua:939: in function `UnitButtonSetHealth'
-- Interface\AddOns\Whammy\Whammy.lua:299: in function `Event_UnitHealth'
-- Interface\AddOns\Whammy\Whammy.lua:621: in function <Interface\AddOns\Whammy\Whammy.lua:576>
-- 
-- Locals: b = W_Button10 {
--  0 = <userdata>
--  magicText = W_Button10_Curse_TextM {
--  }
--  healthAlertBottomTexture = W_Button10HealthAlert_Bottom_Texture {
--  }
--  rangeText = W_Button10_RangeText {
--  }
--  poisonText = W_Button10_Curse_TextP {
--  }
--  curseText = W_Button10_Curse_TextC {
--  }
--  threatTexture = W_Button10_Threat {
--  }
--  diseaseText = W_Button10_Curse_TextD {
--  }
--  healthAlertLeftTexture = W_Button10HealthAlert_Left_Texture {
--  }
--  curseTexture = W_Button10_CurseTop {
--  }
--  unitName = W_Button10_UnitName {
--  }
--  healthAlertTopTexture = W_Button10HealthAlert_Top_Texture {
--  }
--  healthAlertRightTexture = W_Button10HealthAlert_Right_Texture {
--  }
--  texture = W_Button10_Texture {
--  }
-- }
-- unitID = "raid11"
-- class = nil
-- classFileName = nil
-- color = nil
-- red = nil
-- green = nil
-- blue = nil
-- alpha = 0.6
-- d = 1
-- isPlayer = nil
-- inRange = false
-- isDead = nil
-- isGhost = nil
-- isConnected = 1
-- (*temporary) = <function> defined =[C]:-1
-- (*temporary) = W_Button10_RangeText {
--  0 = <userdata>
-- }
-- (*temporary) = "Rng"
-- (*temporary) = <userdata>
-- (*temporary) = nil
-- (*temporary) = nil
-- (*temporary) = "attempt to index local 'color' (a nil value)"


-- function W.GetClassColors( englishClass )
-- 	local r, g, b;
-- 	
-- 	if ( englishClass == "DEATHKNIGHT") then
-- 		r = .77; g = .12; b = .23;
-- 	elseif ( englishClass == "DRUID" ) then
-- 		r = 1.0; g = .49; b = .04;
-- 	elseif ( englishClass == "HUNTER" ) then
-- 		r = .67; g = .83; b = .45;
-- 	elseif ( englishClass == "MAGE" ) then
-- 		r = .41; g = .80; b = .94;
-- 	elseif ( englishClass == "PALADIN" ) then
-- 		r = .96; g = .55; b = .73;	
-- 	elseif ( englishClass == "PRIEST" ) then
-- 		r = 1.0; g = 1.0; b = 1.0;
-- 	elseif ( englishClass == "ROGUE" ) then
-- 		r = 1.0; g = .96; b = .41;
-- 	elseif ( englishClass == "SHAMAN" ) then
-- 		r = .14; g = .35; b = 1.0;
-- 	elseif ( englishClass == "WARLOCK" ) then
-- 		r = .58; g = .51; b = .79;
-- 	elseif ( englishClass == "WARRIOR" ) then
-- 		r = .78; g = .61; b = .43;
-- 	end
-- 
-- 	if (r == nil) then
-- 		W.ShowMessage("GetClassColors: Error. No R "..englishClass)
-- 	elseif (g == nil) then
-- 		W.ShowMessage("GetClassColors: Error. No G"..englishClass)
-- 	elseif ( b == nil) then
-- 		W.ShowMessage("GetClassColors: Error. No B"..englishClass)
-- 	else
-- 		return r, g, b;
-- 	end
-- end


--called when we assign a new unit to a unitbutton, to make their name fit in the width of the button, or when we change button size
function W.UnitButtonSetNameWidth( b )
	
	local buttonWidth = b:GetWidth(); --the width of the button
	local stringWidth = b.unitName:GetStringWidth(); --the width of the unit name

	if ( stringWidth > buttonWidth ) then --if the unit name is wider than the button, then truncate the unit name
		local newStringWidth = buttonWidth * .95 -- the new string width will be 95% of the button width
		b.unitName:SetWidth( newStringWidth );
	else --the unit name fit within the box so reset the string width value
		b.unitName:SetWidth( buttonWidth );
	end
end

function W.UnitButtonSetThreat( b )
	--W.ShowMessage("UnitButtonSetThreat "..b:GetName() )
	
	W.UnitButtonClearThreat();
	
	b.threatTexture:SetTexture( 0, 0, 0, 1)-- indicate this button has threat
	W_gOldThreatButton = b; --keep track of which one this is

end


--
-- Handle Unit Button Mouse
--
function W.UnitButtonMouseDown( b, mouseButton) --MouseDown and up highlight the unit name when clicked. arg1 is the LeftButton, etc. unused
	--W.ShowMessage("UnitButton_MouseDown "..b:GetName().."  "..mouseButton);
	--mousebutton will be LeftButton, MiddleButton, RightButton, Button4, Button5
	
	if ( b:IsEnabled() ) then
		local unitName = b.unitName:GetText();
		b.unitName:SetFontObject(W_kFontNameHighlight)
		if ( W.UnitButtonCheckForAction( b, mouseButton ) ~= true ) then --if there are no actions for this unit button, then see if we want to display a dropdown
			if ( W_Options.ShowDropdown ) then --if we want to show the drop down, validate unit and show it
				W.UnitButtonShowMenu( b );
			end
		end
	end
end

function W.UnitButtonShowMenu( b ) --if the button has a unit assigned but nothing in the mouse button action then show a menu
	--W.ShowMessage("     showing dropdown");
	local menuFrame = CreateFrame("Frame", "W_UnitButtonDropDown", b, "UIDropDownMenuTemplate")
	local text, func, menuList;
	local unitID = b:GetAttribute("unit"); --which unitID is associated with this button
	local isInspectDisabled, isTradeDisabled, isChatDisabled;
	
	if ( not UnitIsUnit("player", unitID)  ) then --only open a drop down menu if we are clicking on a unit button that is not "player"
		if ( CheckInteractDistance( unitID, 1) ) then isInspectDisabled = 1; end;
		if ( CheckInteractDistance( unitID, 2) ) then isTradeDisabled = 2; end;
		if ( CheckInteractDistance( unitID, 4) ) then isFollowDisabled = 1; end;
		
		local menuList = {
			{
				text = W_kUnitButtonMenuTrade,
				func = function() W.UnitButtonHandleMenu("trade", unitID) end,
				disabled = isTradeDisabled,
			},
			{
				text = W_kUnitButtonMenuWhisper,
				func = function() W.UnitButtonHandleMenu("whisper", unitID) end,
				disabled = nil,
			},
			{
				text = W_kUnitButtonMenuInspect,
				func = function() W.UnitButtonHandleMenu("inspect", unitID) end,
				disabled = isInspectDisabled,
			},
			{
				text = W_kUnitButtonMenuFollow,
				func = function() W.UnitButtonHandleMenu("follow", unitID) end,
				disabled = isFollowDisabled,
			},
			{
				text = W_kUnitButtonMenuCancel,
				func = function() W.UnitButtonHandleMenu("cancel", unitID) end,
				disabled = nil,
			}
		}
	
		EasyMenu( menuList, menuFrame, b, 0, 0, "MENU", 1 )
		
	end
end

function W.UnitButtonHandleMenu( theMenuItem, unitID )
	--W.ShowMessage(theMenuItem)
	if ( theMenuItem == "trade" ) then		
		InitiateTrade( unitID ); --1 = Inspect, 28 yards  2 = Trade, 11.11 yards  3 = Duel, 9.9 yards  4 = Follow, 28 yards
	elseif ( theMenuItem == "whisper" ) then
		local unitName = UnitName( unitID )
		ChatFrame_SendTell( unitName );
	elseif ( theMenuItem == "inspect" ) then
		InspectUnit( unitID )
	elseif ( theMenuItem == "follow" ) then
		FollowUnit( unitID )
	elseif ( theMenuItem == "cancel" )	then
		CloseDropDownMenus();
	end
	
end

--check to see if there is an action corresponding to the mouse click, if not, then show a drop down menu
function W.UnitButtonCheckForAction( b, mouseButton )
	--W.ShowMessage("Checking for action for button "..b:GetName() )
	--mousebutton will be LeftButton, MiddleButton, RightButton, Button4, Button5
	
	local keyPress, mouseButtonNum
	local hasAction;
	
	if ( IsLeftShiftKeyDown() or IsRightShiftKeyDown() ) then
		keyPress = "shift"
	elseif ( IsLeftControlKeyDown() or IsRightControlKeyDown() ) then
		keyPress = "ctrl"
	elseif ( IsLeftAltKeyDown() or IsRightAltKeyDown() ) then
		keyPress = "alt"
	else
		keyPress = ""
	end
	
	if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
		keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
	end
	
	if ( mouseButton == "LeftButton" ) then
		mouseButtonNum = 1
	elseif ( mouseButton == "MiddleButton" ) then
		mouseButtonNum = 3
	elseif ( mouseButton == "RightButton" ) then
		mouseButtonNum = 2
	elseif ( mouseButton == "Button4" ) then
		mouseButtonNum = 4;
	elseif ( mouseButton == "Button5" ) then
		mouseButtonNum = 5;
	end
	
	local x = (keyPress.."type"..mouseButtonNum);
	local a = b:GetAttribute(x)
	
	if ( a and a ~="stop" ) then
		hasAction = true;
		--W.ShowMessage(" button has  action ")
	else
		hasAction = false;
		--W.ShowMessage(" button no action ")
	end
	
	return hasAction;
end

function W.UnitButtonClearThreat() --called when player targets something new, need to reset the threat indicator
	--W.ShowMessage("UnitButtonClearThreat")
	if ( W_gOldThreatButton ) then --if someone had threat
		--W.ShowMessage("Clearing threat on "..W_gOldThreatButton:GetName() )
		W_gOldThreatButton.threatTexture:SetTexture( nil ); --clear their threat
		W_gOldThreatButton = nil;
	end
end

-- function W.UnitButtonSetDisconnected( unitID )
-- 	if ( unitID ) then
-- 		local b = W.GetButtonFromUnitID( unitID )
-- 		b.texture:SetTexture( 0,0,0,.2)
-- 	end
-- 
-- end

function W.UnitButtonMouseUp(b) -- unhighlight the name
	if ( b:IsEnabled() ) then
		local unitID = b:GetAttribute("unit"); --get the name of the unit assigned to this button
		if ( unitID ) then
			if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
				b.unitName:SetFontObject(W_kFontNamePlayer)
			else --someone else in the party or raid
				b.unitName:SetFontObject(W_kFontNameOthers)
			end
		end
	end
end

-- End Handle Unit Button Mouse


--
-- MISC functions from here down
--

function W.IsUnitIDPlayerPartyRaid( unitID ) --veryify the unitID is formatted partyx or raidxx. Used to eliminate pets from health checking/curse checking
	if ( string.find(unitID, "party%d", 1) or string.find(unitID, "raid%d", 1) or string.find(unitID, "player") ) then
		return true
	else
		return false
	end
end

function W.GetPlayerGroupStatus()
	--this will return kPlayerAlone, kPlayerInParty or kPlayerInRaid
	
	local numGroupMembers = GetNumGroupMembers();
	local playerGroupStatus;
	
	if ( IsInGroup() ) then --player is in some kind of group
		if ( IsInRaid() ) then
			playerGroupStatus = kPlayerInRaid;
		else
			playerGroupStatus = kPlayerInParty;
		end
	else --player is alone
		playerGroupStatus = kPlayerAlone;
	end
	
	-- if (numGroupMembers == 0) then
	-- 	playerGroupStatus = kPlayerAlone;
	-- else
	-- 	if ( UnitInRaid("player") ) then
	-- 		playerGroupStatus = kPlayerInRaid;
	-- 	else
	-- 		playerGroupStatus = kPlayerInParty;
	-- 	end
	-- end
	
	return playerGroupStatus
end

function W.GetButtonFromUnitID( unitID )
	local f = WhammyFrame;
	local b = f.unitIDArray[unitID];
	
	return b
end

-- Display a message in the chat frame
function W.ShowMessage(aMessage)
	local r,g,b,a
	r = 0;
	g = 1;
	b = 0;
	a = .5;
	
	if ( aMessage ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: " ..aMessage, r, g, b, a);
	else
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: nil message");
	end
end



-- handle our one and only slash command. msg is what the user typed i.e. /whammy msg
function Whammy_Slash_Command_Handler(msg)
	--W.ShowMessage("slash handler");
	local frame = WhammyFrame;
	if ( frame ) then
		if ( msg ) then
			if ( msg == "reset" ) then --user wants to reset whammy - reset position, buttons etc.
				W.ShowMessage("Whammy Reset");
				WC.Reset_Whammy();
			elseif (msg=="show config" or msg=="config" ) then
				WC.Config_Frame_Toggle();
			end
		else
			if ( frame:IsVisible() ) then
				frame:Hide();
				W_Options.ShowWhammy = false;
				if ( WhammyConfigFrame:IsVisible() ) then
					WhammyConfigFrame:Hide();
				end
			else
				frame:Show();
				W_Options.ShowWhammy = true;
			end
		end
	end
end


function W.Info_Button_Click() -- show/hide the info frame when the user clicks the ? button
	local frame = WhammyInfoFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
end

--
-- When the info window loads, set the text that should appear
--
function W.Info_OnLoad(self)
	--W.ShowMessage("W.Info_OnLoad");
	local infoText
	local helpFrameWidth = 440;
	
	WhammyInfoFrame:SetWidth( helpFrameWidth ); --adjust the size of the frame
	WhammyInfoFrame_Heading:SetText("Whammy "..WhammyVers);
	
	-- note that the height of the info frame will adjust around the height of the text. Did this for localization purposes in case another language takes
	--    more or less space to say the same thing
	local t = WhammyInfoFrame_Help_Text;
	
	t:SetPoint("TOPLEFT", "WhammyInfoFrame", 10, -35); --set the starting position of the help text
	t:SetWidth( helpFrameWidth - 15); --set the width of the text
	t:SetText( W_kInfoFrameHelpText ); --set the text from the static text
	local textHeight = t:GetHeight(); --get the height of the text so we can adjust the frame height around it
	WhammyInfoFrame:SetHeight( textHeight + 80 ) --set the frame height to the height of the text + space for the heading area
end




